@@ -413,8 +413,12 @@ set_ui (GtkHTMLEditImageProperties *d)
 static void
 changed_template (GtkWidget *w, GtkHTMLEditImageProperties *d)
 {
+	gint oldtemplate = d->template;
 	d->template = g_list_index (GTK_MENU_SHELL (w)->children, gtk_menu_get_active (GTK_MENU (w)));
 
+	if (d->template == oldtemplate)
+		return;
+
 	d->border = image_templates [d->template].border;
 	d->align = image_templates [d->template].align;
 	d->padh = image_templates [d->template].padh;
@@ -507,8 +511,8 @@ set_size_all (HTMLObject *o, HTMLEngine *e, GtkHTMLEditImageProperties *d)
 	}
 }
 
-static gboolean
-set_size (GtkHTMLEditImageProperties *d)
+static
+load_done (GtkHTML *html, GtkHTMLEditImageProperties *d)
 {
 	if (d->sample->engine->clue) {
 		html_object_forall (d->sample->engine->clue, d->sample->engine, (HTMLObjectForallFunc) set_size_all, d);
@@ -523,8 +527,6 @@ image_url_requested (GtkHTML *html, const gchar *url, GtkHTMLStream *handle, Gtk
 
 	location = get_location (d);
 	url_requested (html, url, handle);
-	if (location && !strcmp (location, url))
-		gtk_idle_add ((GtkFunction) set_size, d);
 	g_free (location);
 }
 
@@ -536,7 +538,7 @@ image_widget (GtkHTMLEditImageProperties *d, gboolean insert)
 	GladeXML *xml;
 	GtkWidget *frame_template;
 
-	xml = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "image_page", NULL);
+	xml = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "image_page", GETTEXT_PACKAGE);
 	if (!xml)
 		g_error (_("Could not load glade file."));
 
@@ -578,6 +580,7 @@ image_widget (GtkHTMLEditImageProperties *d, gboolean insert)
 
 	gtk_container_add (GTK_CONTAINER (d->frame_sample), sample_frame (&d->sample));
 	g_signal_handlers_disconnect_matched (d->sample, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, G_CALLBACK (url_requested), NULL);
+	g_signal_connect (GTK_OBJECT (d->sample), "load_done", G_CALLBACK (load_done), d);
 	g_signal_connect (GTK_OBJECT (d->sample), "url_requested", G_CALLBACK (image_url_requested), d);
 
 	d->entry_url = glade_xml_get_widget (xml, "entry_image_url");
@@ -716,7 +719,7 @@ insert_or_apply (GtkHTMLControlData *cd, gpointer get_data, gboolean insert)
 		location = get_location (d);
 		html_image_edit_set_url (image, location);
 		g_free (location);
-		html_image_set_alt (image, d->url);
+		html_image_set_alt (image, d->alt);
 
 		url = d->url;
 		target = NULL;
