@@ -22,9 +22,9 @@ Boston, MA 02111-1307, USA.
 
 #include <config.h>
 #include <string.h>
+#include <locale.h>
 #include <gdk/gdkx.h>
 #include <libgnome/gnome-i18n.h>
-#include <gal/util/e-iconv.h>
 #include "gtkhtml.h"
 #include "gtkhtml-properties.h"
 #include "htmlfontmanager.h"
@@ -34,51 +34,60 @@ Boston, MA 02111-1307, USA.
 
 #define STRINGIZE(x) #x
 
+/* this function is reworked gal:e-iconv.c:e_iconv_locale_language */
+static gchar *
+get_locale_language ()
+{
+	gchar *locale;
+	gchar *locale_language = NULL;
+	
+	locale = setlocale (LC_ALL, NULL);
+	if (locale && strcmp (locale, "C") && strcmp (locale, "POSIX")) {
+		char *codeset, *lang;
+	
+		if ((codeset = strchr (locale, '.')))
+			lang = g_strndup (locale, codeset - locale);
+		else
+			lang = g_strdup (locale);
+	
+		/* validate the language */
+		if (strlen (lang) >= 2) {
+			if (lang[2] == '-' || lang[2] == '_') {
+				/* canonicalise the lang */
+				lang [0] = g_ascii_tolower (lang [0]);
+				lang [1] = g_ascii_tolower (lang [1]);
+			
+				/* validate the country code */
+				if (strlen (lang + 3) > 2) {
+				        /* invalid country code */
+					lang[2] = '\0';
+				} else {
+					lang[2] = '-';
+					lang [3] = g_ascii_toupper (lang [3]);
+					lang [4] = g_ascii_toupper (lang [4]);
+				}
+			} else if (lang[2] != '\0') {
+				/* invalid language */
+				g_free (lang);
+				lang = NULL;
+			}
+		
+			locale_language = lang;
+		} else {
+			/* invalid language */
+			g_free (lang);
+		}
+	}
+
+	return locale_language;
+}
+
 GtkHTMLClassProperties *
 gtk_html_class_properties_new (GtkWidget *widget)
 {
 	GtkHTMLClassProperties *p = g_new0 (GtkHTMLClassProperties, 1);
-	PangoFontDescription *var_desc, *fixed_desc; 
-	char *fixed_name = NULL;
-	gint var_size, fixed_size = 0;
 
-	var_desc = widget->style->font_desc;
-	gtk_widget_style_get (widget,
-			      "fixed_font_name",   &fixed_name,
-			      NULL);
-       
-	var_size = pango_font_description_get_size (var_desc);
-
-	if (fixed_name) {
-		fixed_desc = pango_font_description_from_string (fixed_name);
-		if (pango_font_description_get_family (fixed_desc)) {
-			fixed_size = PANGO_PIXELS (pango_font_description_get_size (fixed_desc));
-			pango_font_description_free (fixed_desc);
-		} else {
-			g_free (fixed_name);
-			fixed_name = NULL;
-		}
-	}
-
-	if (!fixed_name) {
-		fixed_name = g_strdup ("Monospace");
-		fixed_desc = pango_font_description_from_string (fixed_name);
-		fixed_size = var_size;
-	}
-
-	/* default values */
-	p->magic_links             = TRUE;
-	p->animations              = TRUE;
-	p->language                = g_strdup (e_iconv_locale_language ());
-	p->live_spell_check        = TRUE;
-
-	p->font_var_print          = g_strdup (pango_font_description_get_family (var_desc));
-	/* printf ("Variable Printing Font: \"%s\"\n", p->font_var_print); */
-	p->font_fix_print          = g_strdup (fixed_name);
-	p->font_var_size_print     = DEFAULT_FONT_SIZE_PRINT;
-	p->font_fix_size_print     = DEFAULT_FONT_SIZE_PRINT;
-	p->font_var_print_points   = FALSE;
-	p->font_fix_print_points   = FALSE;
+	p->language = get_locale_language ();
 
 	return p;
 }
@@ -86,16 +95,10 @@ gtk_html_class_properties_new (GtkWidget *widget)
 void
 gtk_html_class_properties_destroy (GtkHTMLClassProperties *p)
 {
+	g_free (p->language);
 	g_free (p);
 }
 
-#define GET(t,x,prop,f,c) \
-        key = g_strconcat (GTK_HTML_GCONF_DIR, x, NULL); \
-        val = gconf_client_get_without_default (client, key, NULL); \
-        if (val) { f; p->prop = c (gconf_value_get_ ## t (val)); \
-        gconf_value_free (val); } \
-        g_free (key);
-
 #define GNOME_SPELL_GCONF_DIR "/GNOME/Spell"
 #define GETSP(t,x,prop,f,c) \
         key = g_strconcat (GNOME_SPELL_GCONF_DIR, x, NULL); \
@@ -112,9 +115,6 @@ gtk_html_class_properties_load (GtkHTMLClassProperties *p, GConfClient *client)
 
 	g_assert (client);
 
-	GET (bool, "/magic_links", magic_links,,);
-	GET (bool, "/animations", animations,,);
-
 	GETSP (string, "/language", language,
 	       g_free (p->language), g_strdup);
 
@@ -129,31 +129,6 @@ gtk_html_class_properties_load (GtkHTMLClassProperties *p, GConfClient *client)
 void
 gtk_html_class_properties_update (GtkHTMLClassProperties *p, GConfClient *client, GtkHTMLClassProperties *old)
 {
-	gchar *key;
-
-	if (p->animations != old->animations)
-		SET (bool, "/animations", animations);
-	if (p->magic_links != old->magic_links)
-		SET (bool, "/magic_links", magic_links);
-	if (p->live_spell_check != old->live_spell_check)
-		SET (bool, "/live_spell_check", live_spell_check);
-
-
-	if (strcmp (p->font_var_print, old->font_var_print))
-		SET (string, "/font_variable_print", font_var_print);
-	if (strcmp (p->font_fix_print, old->font_fix_print))
-		SET (string, "/font_fixed_print", font_fix_print);
-	if (p->font_var_print_points != old->font_var_print_points)
-		SET (bool, "/font_variable_print_points", font_var_print_points);
-	if (p->font_fix_print_points != old->font_fix_print_points)
-		SET (bool, "/font_fixed_print_points", font_fix_print_points);
-	if (p->font_var_size_print != old->font_var_size_print || p->font_var_print_points != old->font_var_print_points)
-		SET (int, "/font_variable_size_print", font_var_size_print);
-	if (p->font_fix_size_print != old->font_fix_size_print || p->font_fix_print_points != old->font_fix_print_points)
-		SET (int, "/font_fixed_size_print", font_fix_size_print);
-	
-	
-	gconf_client_suggest_sync (client, NULL);
 }
 
 /* enums */
