@@ -281,15 +281,17 @@ set_max_height (HTMLObject *o, HTMLPainter *painter, gint height)
 	HTMLClue *clue = HTML_CLUE (o);
 	HTMLObject *obj;
 
-	for (obj = HTML_CLUE (o)->head; obj != 0; obj = obj->next) {
-		html_object_set_max_height (obj, painter, height);
-		if (clue->valign == HTML_VALIGN_MIDDLE)
-			obj->y = obj->y + ((height - o->ascent) / 2);
-		else if (clue->valign == HTML_VALIGN_BOTTOM)
-			obj->y = obj->y + height - o->ascent;
-	}
+	if (o->ascent < height) {
+		for (obj = HTML_CLUE (o)->head; obj != 0; obj = obj->next) {
+			html_object_set_max_height (obj, painter, height);
+			if (clue->valign == HTML_VALIGN_MIDDLE)
+				obj->y += (height - o->ascent)/2;
+			else if (clue->valign == HTML_VALIGN_BOTTOM)
+				obj->y += height - o->ascent;
+		}
 
-	o->ascent = height;
+		o->ascent = height;
+	}
 }
 
 static void
@@ -306,7 +308,7 @@ reset (HTMLObject *clue)
 }
 
 static gboolean
-calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
+html_clue_real_calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 {
 	gboolean changed;
 
@@ -630,7 +632,7 @@ html_clue_class_init (HTMLClueClass *klass,
 	object_class->draw = draw;
 	object_class->set_max_height = set_max_height;
 	object_class->reset = reset;
-	object_class->calc_size = calc_size;
+	object_class->calc_size = html_clue_real_calc_size;
 	object_class->calc_preferred_width = calc_preferred_width;
 	object_class->calc_min_width = calc_min_width;
 	object_class->check_point = check_point;
