@@ -4,6 +4,7 @@
    Copyright (C) 1997 Martin Jones (mjones@kde.org)
    Copyright (C) 1997 Torben Weis (weis@kde.org)
    Copyright (C) 1999, 2000 Helix Code, Inc.
+   Copyright (C) 2000, 2001, 2002, 2003 Ximian, Inc.
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -24,67 +25,54 @@
 #include <config.h>
 #include <string.h> /* strcmp */
 #include <stdlib.h>
-#include "gtkhtml.h"
 #include "gtkhtml-compat.h"
-#include "gtkhtml-properties.h"
 
-#include <gal/unicode/gunicode.h>
 #include "htmlcolor.h"
 #include "htmlcolorset.h"
-#include "htmlengine.h"
 #include "htmlentity.h"
+#include "htmltext.h"
+#include "htmltextslave.h"
 #include "htmlpainter.h"
-#include "htmlgdkpainter.h"
-#include "htmlplainpainter.h"
-#include "htmlprinter.h"
 
 
 /* Convenience macro to extract the HTMLPainterClass from a GTK+ object.  */
 #define HP_CLASS(obj)					\
-	HTML_PAINTER_CLASS (GTK_OBJECT (obj)->klass)
-
-#define HTML_ALLOCA_MAX 2048
+	HTML_PAINTER_CLASS (G_OBJECT_GET_CLASS (obj))
 
 /* Our parent class.  */
-static GtkObjectClass *parent_class = NULL;
-
-static gint calc_text_bytes_delta (const gchar *text, gint len, gint line_offset, gint *translated_len, gboolean tabs);
-static gint translate_text_special_chars (const gchar *text, gchar *translated, gint len, gint line_offset, gboolean tabs);
+static GObjectClass *parent_class = NULL;
 
 
-/* GtkObject methods.  */
+/* GObject methods.  */
 
 static void
-finalize (GtkObject *object)
+finalize (GObject *object)
 {
 	HTMLPainter *painter;
 
 	painter = HTML_PAINTER (object);
+	html_font_manager_finalize (&painter->font_manager);
+
 	html_colorset_destroy (painter->color_set);
+	g_free (painter->font_face);
 
 	/* FIXME ownership of the color set?  */
 
-	(* GTK_OBJECT_CLASS (parent_class)->finalize) (object);
+	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
+
+	if (painter->widget) {
+		g_object_unref (painter->widget);
+		painter->widget = NULL;
+	}
 }
 
 
-#ifndef GAL_NOT_SLOW
-/* NOTE see htmltextslave.c for an explanation and the h_utf8 decls */
-gint h_utf8_pointer_to_offset (const gchar *str, const gchar *pos);
-gint h_utf8_strlen (const gchar *p, gint max);
-gchar *h_utf8_offset_to_pointer  (const gchar *str, gint         offset);
-
-#define g_utf8_strlen h_utf8_strlen
-#define g_utf8_offset_to_pointer h_utf8_offset_to_pointer
-#define g_utf8_pointer_to_offset h_utf8_pointer_to_offset
-#endif
-
 #define DEFINE_UNIMPLEMENTED(method)						\
 	static gint								\
-	method##_unimplemented (GtkObject *obj)					\
+	method##_unimplemented (GObject *obj)					\
 	{									\
 		g_warning ("Class `%s' does not implement `" #method "'\n",	\
-			   gtk_type_name (GTK_OBJECT_TYPE (obj)));		\
+			   g_type_name (G_TYPE_FROM_INSTANCE (obj)));		\
 		return 0;							\
 	}
 
@@ -98,10 +86,8 @@ DEFINE_UNIMPLEMENTED (unref_font)
 DEFINE_UNIMPLEMENTED (alloc_color)
 DEFINE_UNIMPLEMENTED (free_color)
 
-DEFINE_UNIMPLEMENTED (calc_ascent)
-DEFINE_UNIMPLEMENTED (calc_descent)
-DEFINE_UNIMPLEMENTED (calc_text_width)
-DEFINE_UNIMPLEMENTED (calc_text_width_bytes)
+DEFINE_UNIMPLEMENTED (calc_text_size)
+DEFINE_UNIMPLEMENTED (calc_text_size_bytes)
 
 DEFINE_UNIMPLEMENTED (set_pen)
 DEFINE_UNIMPLEMENTED (get_black)
@@ -125,33 +111,40 @@ DEFINE_UNIMPLEMENTED (get_pixel_size)
 DEFINE_UNIMPLEMENTED (get_page_width)
 DEFINE_UNIMPLEMENTED (get_page_height)
 
-DEFINE_UNIMPLEMENTED (get_font_manager_id)
-
-
 static void
-init (GtkObject *object, HTMLPainterClass *real_klass)
+html_painter_init (GObject *object, HTMLPainterClass *real_klass)
 {
 	HTMLPainter *painter;
 
 	painter = HTML_PAINTER (object);
 	painter->color_set = html_colorset_new (NULL);
 
+	html_font_manager_init (&painter->font_manager, painter);
 	painter->font_style = GTK_HTML_FONT_STYLE_DEFAULT;
 	painter->font_face = NULL;
-
-	painter->magnification = 1.0;
-	painter->mag_fm = NULL;
+	painter->widget = NULL;
 }
 
 static void
-class_init (GtkObjectClass *object_class)
+html_painter_real_set_widget (HTMLPainter *painter, GtkWidget *widget)
+{
+	if (painter->widget)
+		g_object_unref (painter->widget);
+	painter->widget = widget;
+	g_object_ref (widget);
+}
+
+static void
+html_painter_class_init (GObjectClass *object_class)
 {
 	HTMLPainterClass *class;
 
 	class = HTML_PAINTER_CLASS (object_class);
 
 	object_class->finalize = finalize;
+	parent_class = g_type_class_ref (G_TYPE_OBJECT);
 
+	class->set_widget = html_painter_real_set_widget;
 	class->begin = (gpointer) begin_unimplemented;
 	class->end = (gpointer) end_unimplemented;
 
@@ -162,10 +155,8 @@ class_init (GtkObjectClass *object_class)
 	class->alloc_color = (gpointer) alloc_color_unimplemented;
 	class->free_color = (gpointer) free_color_unimplemented;
 
-	class->calc_ascent = (gpointer) calc_ascent_unimplemented;
-	class->calc_descent = (gpointer) calc_descent_unimplemented;
-	class->calc_text_width = (gpointer) calc_text_width_unimplemented;
-	class->calc_text_width_bytes = (gpointer) calc_text_width_bytes_unimplemented;
+	class->calc_text_size = (gpointer) calc_text_size_unimplemented;
+	class->calc_text_size_bytes = (gpointer) calc_text_size_bytes_unimplemented;
 
 	class->set_pen = (gpointer) set_pen_unimplemented;
 	class->get_black = (gpointer) get_black_unimplemented;
@@ -189,43 +180,31 @@ class_init (GtkObjectClass *object_class)
 
 	class->get_page_width  = (gpointer) get_page_width_unimplemented;
 	class->get_page_height = (gpointer) get_page_height_unimplemented;
-
-	class->get_font_manager_id = (gpointer) get_font_manager_id_unimplemented;
-
-	parent_class = gtk_type_class (gtk_object_get_type ());
 }
 
-
-GtkType
+GType
 html_painter_get_type (void)
 {
-	static GtkType type = 0;
+	static GType html_painter_type = 0;
 
-	if (type == 0) {
-		static const GtkTypeInfo info = {
-			"HTMLPainter",
-			sizeof (HTMLPainter),
+	if (html_painter_type == 0) {
+		static const GTypeInfo html_painter_info = {
 			sizeof (HTMLPainterClass),
-			(GtkClassInitFunc) class_init,
-			(GtkObjectInitFunc) init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL,
+			NULL,
+			NULL,
+			(GClassInitFunc) html_painter_class_init,
+			NULL,
+			NULL,
+			sizeof (HTMLPainter),
+			1,
+			(GInstanceInitFunc) html_painter_init,
 		};
-
-		type = gtk_type_unique (GTK_TYPE_OBJECT, &info);
+		html_painter_type = g_type_register_static (G_TYPE_OBJECT, "HTMLPainter", &html_painter_info, 0);
 	}
 
-	return type;
+	return html_painter_type;
 }
 
-HTMLPainter *
-html_painter_new (void)
-{
-	return gtk_type_new (html_painter_get_type ());
-}
-
-
 /* Functions to begin/end a painting process.  */
 
 void
@@ -307,160 +286,52 @@ html_painter_set_font_face (HTMLPainter *painter,
 	}
 }
 
-static HTMLFont *
-get_html_font (HTMLPainter *painter, HTMLFontFace *face, GtkHTMLFontStyle style)
-{
-	HTMLEngineClass *ec = gtk_type_class (html_engine_get_type ());
-	HTMLFontManager *fm;
-
-	fm = painter->mag_fm ? painter->mag_fm : &ec->font_manager [html_painter_get_font_manager_id (painter)];
-
-	return html_font_manager_get_font (fm, face, style);
-}
-
-HTMLFont *
-html_painter_get_html_font (HTMLPainter *painter, HTMLFontFace *face, GtkHTMLFontStyle style)
-{
-	return get_html_font (painter, face, style);
-}
-
 gpointer
 html_painter_get_font (HTMLPainter *painter, HTMLFontFace *face, GtkHTMLFontStyle style)
 {
-	HTMLFont *font = get_html_font (painter, face, style);
+	HTMLFont *font;
 
+	font = html_font_manager_get_font (&painter->font_manager, face, style);
 	return font ? font->data : NULL;
 }
 
-guint
-html_painter_calc_ascent (HTMLPainter *painter,
-			  GtkHTMLFontStyle font_style,
-			  HTMLFontFace *face)
+void
+html_painter_calc_text_size (HTMLPainter *painter,
+			     const gchar *text,
+			     guint len, HTMLTextPangoInfo *pi, PangoAttrList *attrs, GList *glyphs, gint start_byte_offset, gint *line_offset,
+			     GtkHTMLFontStyle font_style,
+			     HTMLFontFace *face,
+			     gint *width, gint *asc, gint *dsc)
 {
-	g_return_val_if_fail (painter != NULL, 0);
-	g_return_val_if_fail (HTML_IS_PAINTER (painter), 0);
-	g_return_val_if_fail (font_style != GTK_HTML_FONT_STYLE_DEFAULT, 0);
+	g_return_if_fail (painter != NULL);
+	g_return_if_fail (HTML_IS_PAINTER (painter));
+	g_return_if_fail (text != NULL);
 
-	return (* HP_CLASS (painter)->calc_ascent) (painter, font_style, face);
-}
+	(* HP_CLASS (painter)->calc_text_size) (painter, text, len, pi, attrs, glyphs, start_byte_offset, font_style, face, width, asc, dsc);
 
-guint
-html_painter_calc_descent (HTMLPainter *painter,
-			   GtkHTMLFontStyle font_style,
-			   HTMLFontFace *face)
-{
-	g_return_val_if_fail (painter != NULL, 0);
-	g_return_val_if_fail (HTML_IS_PAINTER (painter), 0);
-	g_return_val_if_fail (font_style != GTK_HTML_FONT_STYLE_DEFAULT, 0);
-
-	return (* HP_CLASS (painter)->calc_descent) (painter, font_style, face);
-}
-
-guint
-html_painter_calc_text_width (HTMLPainter *painter,
-			      const gchar *text,
-			      guint len, gint *line_offset,
-			      GtkHTMLFontStyle font_style,
-			      HTMLFontFace *face)
-{
-	guint width;
-	gchar *tmp = NULL;
-	gchar *translated;
-	gint translated_len;
-	gint tmp_len;
-	
-	g_return_val_if_fail (painter != NULL, 0);
-	g_return_val_if_fail (HTML_IS_PAINTER (painter), 0);
-	g_return_val_if_fail (text != NULL, 0);
-	g_return_val_if_fail (font_style != GTK_HTML_FONT_STYLE_DEFAULT, 0);
-
-	tmp_len =  (g_utf8_offset_to_pointer (text,len) - text) + calc_text_bytes_delta (text, len, *line_offset, &translated_len,
-											 *line_offset != -1) + 1;
-
-	if (tmp_len > HTML_ALLOCA_MAX)
-		tmp = translated = g_malloc (tmp_len);
-	else 
-		translated = alloca (tmp_len);
-
-	*line_offset = translate_text_special_chars (text, translated, len, *line_offset, *line_offset != -1);
-
-	width = (* HP_CLASS (painter)->calc_text_width) (painter, translated, translated_len, font_style, face);
-	
-	g_free (tmp);
-	return width;
-}
-
-static gint
-correct_width (const gchar *text, guint bytes_len, gint *lo, HTMLFont *font)
-{
-	gint delta = 0;
-	gunichar uc;
-	const gchar *s, *end = text + bytes_len;
-	gint skip, line_offset = *lo;
-	gboolean tabs = *lo != -1;
-
-	if (!tabs) {
-		if (font->space_width == font->nbsp_width) {
-			if (font->space_width == font->tab_width) {
-				return 0;
-			} else {
-				while (text < end) {
-					if (*text == '\t')
-						delta += font->space_width - font->tab_width;
-					text ++;
-				}
-
-				return delta;
-			}
-		}
+	if (line_offset) {
+		gint tabs;
+		*width += (html_text_text_line_length (text, line_offset, len, &tabs) - len + tabs)*html_painter_get_space_width (painter, font_style, face);
 	}
-
-	s = text;
-	while (s < end && (uc = g_utf8_get_char (s))) {
-		switch (uc) {
-		case ENTITY_NBSP:
-			line_offset ++;
-			delta += font->space_width - font->nbsp_width;
-			break;
-		case '\t':
-			if (tabs) {
-				skip = 8 - (line_offset % 8);
-				line_offset += skip;
-				delta += skip * font->space_width - font->tab_width;
-			} else {
-				delta += font->space_width - font->tab_width;
-				line_offset ++;
-			}
-			break;
-		default:
-			line_offset ++;
-		}
-		s = g_utf8_next_char (s);
-	}
-
-	if (tabs)
-		*lo = line_offset;
-
-	return delta;
 }
 
-guint
-html_painter_calc_text_width_bytes (HTMLPainter *painter,
-				    const gchar *text,
-				    guint bytes_len, gint *line_offset,
-				    HTMLFont *font, GtkHTMLFontStyle style)
+void
+html_painter_calc_text_size_bytes (HTMLPainter *painter,
+				   const gchar *text,
+				   guint bytes_len, HTMLTextPangoInfo *pi, PangoAttrList *attrs, GList *glyphs, gint start_byte_offset, gint *line_offset,
+				   HTMLFont *font, GtkHTMLFontStyle style,
+				   gint *width, gint *asc, gint *dsc)
 {
-	guint width;
+	g_return_if_fail (painter != NULL);
+	g_return_if_fail (HTML_IS_PAINTER (painter));
+	g_return_if_fail (text != NULL);
+	g_return_if_fail (style != GTK_HTML_FONT_STYLE_DEFAULT);
 
-	g_return_val_if_fail (painter != NULL, 0);
-	g_return_val_if_fail (HTML_IS_PAINTER (painter), 0);
-	g_return_val_if_fail (text != NULL, 0);
-	g_return_val_if_fail (style != GTK_HTML_FONT_STYLE_DEFAULT, 0);
-
-	width = (* HP_CLASS (painter)->calc_text_width_bytes) (painter, text, bytes_len, font, style);
-	width += correct_width (text, bytes_len, line_offset, font);
-
-	return width;
+	(* HP_CLASS (painter)->calc_text_size_bytes) (painter, text, bytes_len, pi, attrs, glyphs, start_byte_offset, font, style, width, asc, dsc);
+	if (line_offset) {
+		gint tabs, len = g_utf8_pointer_to_offset (text, text + bytes_len);
+		*width += (html_text_text_line_length (text, line_offset, len, &tabs) - len + tabs)*font->space_width;
+	}
 }
 
 /* The actual paint operations.  */
@@ -498,127 +369,90 @@ html_painter_draw_rect (HTMLPainter *painter,
 	(* HP_CLASS (painter)->draw_rect) (painter, x, y, width, height);
 }
 
-static gint
-calc_text_bytes_delta (const gchar *text, gint len, gint line_offset, gint *translated_len, gboolean tabs)
+void
+html_replace_tabs (const gchar *text, gchar *translated, guint bytes)
 {
-	gunichar uc;
-	const gchar *s;
-	gint delta, skip, current_len;
+	const gchar *t, *tab;
+	gchar *tt;
 
-	current_len = 0;
-	delta = 0;
-	*translated_len = 0;
-	s = text;
-	while (s && (uc = g_utf8_get_char (s)) && current_len < len) {
-		switch (uc) {
-		case ENTITY_NBSP:
-			delta --;
-			(*translated_len) ++;
-			line_offset ++;
-			break;
-		case '\t':
-			if (tabs) {
-				skip = 8 - (line_offset % 8);
-				delta += skip - 1;
-				line_offset += skip;
-				(*translated_len) += skip;
-			} else {
-				(*translated_len) ++;
-				line_offset ++;
-			}
-			break;
-		default:
-			(*translated_len) ++;
-			line_offset ++;
-		}
-		current_len ++;
-		s = g_utf8_next_char (s);
-	}
+	t = text;
+	tt = translated;
 
-	return delta;
+	do {
+		tab = memchr (t, (unsigned char) '\t', bytes - (t - text));
+		if (tab) {
+			strncpy (tt, t, tab - t);
+			tt += tab - t;
+			*tt = ' ';
+			tt ++;
+			t = tab + 1;
+		} else
+			strncpy (tt, t, bytes - (t - text));
+	} while (tab);
 }
 
-static inline void
-put_last (const gchar *s, const gchar **ls, gchar **translated)
+static inline GList *
+shift_items (GList *items, gint byte_offset)
 {
-	if (*ls)
-		for (; *ls < s; (*ls) ++, (*translated) ++)
-			**translated = **ls;
+	if (items) {
+		PangoItem *item;
+
+		while (items && (item = (PangoItem *) items->data) && item->offset + item->length <= byte_offset)
+			items = items->next;
+	}
+
+	return items;
 }
 
-static gint
-translate_text_special_chars (const gchar *text, gchar *translated, gint len, gint line_offset, gboolean tabs)
+static inline GList *
+shift_glyphs (GList *glyphs, gint len)
 {
-	gunichar uc;
-	const gchar *s, *ls;
-	gint skip, current_len;
+	if (glyphs) {
+		PangoGlyphString *str;
 
-	current_len = 0;
-	s = text;
-	ls = NULL;
-	while (s && (uc = g_utf8_get_char (s)) && current_len < len) {
-		put_last (s, &ls, &translated);
-		switch (uc) {
-		case ENTITY_NBSP:
-			*translated = ' ';
-			translated ++;
-			line_offset ++;
-			ls = NULL;
-			break;
-		case '\t':
-			if (tabs) {
-				skip = 8 - (line_offset % 8);
-				line_offset += skip;
-				for (; skip; skip --, translated ++)
-					*translated = ' ';
-			} else {
-				*translated = ' ';
-				translated ++;
-				line_offset ++;
-			}
-			ls = NULL;
-			break;
-		default:
-			ls = s;
-			line_offset ++;
+		while (glyphs && (str = (PangoGlyphString *) glyphs->data) && len > 0) {
+			len -= str->num_glyphs;
+			glyphs = glyphs->next->next;
 		}
-		current_len ++;
-		s = g_utf8_next_char (s);
 	}
-	put_last (s, &ls, &translated);
-	*translated = 0;
 
-	return line_offset;
+	return glyphs;
 }
 
 gint
-html_painter_draw_text (HTMLPainter *painter,
-			gint x, gint y,
-			const gchar *text, gint len, gint line_offset)
+html_painter_draw_text (HTMLPainter *painter, gint x, gint y,
+			const gchar *text, gint len, HTMLTextPangoInfo *pi, PangoAttrList *attrs, GList *glyphs, gint start_byte_offset, gint line_offset)
 {
-	gchar *translated;
-	gchar *tmp = NULL;
-	gint translated_len;
-	gint tmp_len;
+	const gchar *tab, *c_text = text;
+	gint bytes, byte_offset = 0;
 
 	g_return_val_if_fail (painter != NULL, line_offset);
 	g_return_val_if_fail (HTML_IS_PAINTER (painter), line_offset);
 
-	tmp_len = (g_utf8_offset_to_pointer (text, len) - text) + calc_text_bytes_delta (text, len, line_offset, &translated_len,
-					       line_offset != -1) + 1;
-	
-	if (tmp_len > HTML_ALLOCA_MAX)
-		tmp = translated = g_malloc (tmp_len);
-	else 
-		translated = alloca (tmp_len);
+	bytes = g_utf8_offset_to_pointer (text, len) - text;
+	while ((tab = memchr (c_text, (unsigned char) '\t', bytes))) {
+		gint c_bytes = tab - c_text;
+		gint c_len = g_utf8_pointer_to_offset (c_text, tab);
+		
+		if (c_bytes)
+			x += (* HP_CLASS (painter)->draw_text) (painter, x, y, c_text, c_len, pi, NULL, glyphs, start_byte_offset + (c_text - text));
+		if (line_offset == -1)
+			x += html_painter_get_space_width (painter, painter->font_style, painter->font_face);
+		else {
+			line_offset += c_len;
+			x += html_painter_get_space_width (painter, painter->font_style, painter->font_face)*(8 - (line_offset % 8));
+			line_offset += 8 - (line_offset % 8);
+		}
+		c_text += c_bytes + 1;
+		bytes -= c_bytes + 1;
+		byte_offset += c_bytes + 1;
+		glyphs = shift_glyphs (glyphs, c_len);
+		len -= c_len + 1;
+	}
 
-	line_offset = translate_text_special_chars (text, translated, len, line_offset, line_offset != -1);
+	(* HP_CLASS (painter)->draw_text) (painter, x, y, c_text, len, pi, attrs, glyphs, start_byte_offset + (c_text - text));
 
-	(* HP_CLASS (painter)->draw_text) (painter, x, y, translated, translated_len);
-
-	g_free (tmp);
-
-	return line_offset;
+	return line_offset + len;
 }
 
 void
@@ -752,33 +586,57 @@ gint
 html_painter_draw_spell_error (HTMLPainter *painter,
 			       gint x, gint y,
 			       const gchar *text,
-			       gint len)
+			       gint len, HTMLTextPangoInfo *pi, GList *glyphs, gint start_byte_offset)
 {
-	return (* HP_CLASS (painter)->draw_spell_error) (painter, x, y, text, len);
+	return (* HP_CLASS (painter)->draw_spell_error) (painter, x, y, text, len, pi, glyphs, start_byte_offset);
 }
 
 HTMLFont *
-html_painter_alloc_font (HTMLPainterClass *pc, gchar *face_name, gdouble size, gboolean points, GtkHTMLFontStyle style)
+html_painter_alloc_font (HTMLPainter *painter, gchar *face_name, gdouble size, gboolean points, GtkHTMLFontStyle style)
 {
-	return (* pc->alloc_font) (face_name, size, points, style);
+	return (* HP_CLASS (painter)->alloc_font) (painter, face_name, size, points, style);
 }
 
 void
-html_painter_ref_font (HTMLPainterClass *pc, HTMLFont *font)
+html_painter_ref_font (HTMLPainter *painter, HTMLFont *font)
 {
-	(* pc->ref_font) (font);
+	(* HP_CLASS (painter)->ref_font) (painter, font);
 }
 
 void
-html_painter_unref_font (HTMLPainterClass *pc, HTMLFont *font)
+html_painter_unref_font (HTMLPainter *painter, HTMLFont *font)
 {
-	(* pc->unref_font) (font);
+	(* HP_CLASS (painter)->unref_font) (painter, font);
 }
 
 guint
 html_painter_get_space_width (HTMLPainter *painter, GtkHTMLFontStyle style, HTMLFontFace *face)
 {
-	return get_html_font (painter, face, style)->space_width;
+	return html_font_manager_get_font (&painter->font_manager, face, style)->space_width;
+}
+
+guint
+html_painter_get_space_asc (HTMLPainter *painter, GtkHTMLFontStyle style, HTMLFontFace *face)
+{
+	return html_font_manager_get_font (&painter->font_manager, face, style)->space_asc;
+}
+
+guint
+html_painter_get_space_dsc (HTMLPainter *painter, GtkHTMLFontStyle style, HTMLFontFace *face)
+{
+	return html_font_manager_get_font (&painter->font_manager, face, style)->space_dsc;
+}
+
+guint
+html_painter_get_block_indent_width (HTMLPainter *painter, GtkHTMLFontStyle style, HTMLFontFace *face)
+{
+	return html_font_manager_get_font (&painter->font_manager, face, style)->indent_width;
+}
+
+guint
+html_painter_get_block_cite_width (HTMLPainter *painter, GtkHTMLFontStyle style, HTMLFontFace *face)
+{
+	return html_font_manager_get_font (&painter->font_manager, face, style)->cite_width;
 }
 
 guint
@@ -799,49 +657,64 @@ html_painter_set_focus (HTMLPainter *p, gboolean focus)
 	p->focus = focus;
 }
 
-HTMLPainterClass *
-html_painter_class_from_id (HTMLFontManagerId id)
+void
+html_painter_set_widget (HTMLPainter *painter, GtkWidget *widget)
 {
-	switch (id) {
-	case HTML_FONT_MANAGER_ID_GDK:
-		return gtk_type_class (html_gdk_painter_get_type ());
-	case HTML_FONT_MANAGER_ID_PLAIN:
-		return gtk_type_class (html_plain_painter_get_type ());
-	case HTML_FONT_MANAGER_ID_PRINTER:
-		return gtk_type_class (html_printer_get_type ());
-	default:
-		return NULL;
-	}
+	return 	(* HP_CLASS (painter)->set_widget) (painter, widget);
 }
 
-HTMLFontManagerId
-html_painter_get_font_manager_id (HTMLPainter *painter)
+HTMLTextPangoInfo *
+html_painter_text_itemize_and_prepare_glyphs (HTMLPainter *painter, PangoFontDescription *desc, const gchar *text, gint bytes, GList **glyphs, PangoAttrList *attrs)
 {
-	return 	(* HP_CLASS (painter)->get_font_manager_id) ();
+	PangoAttribute *attr;
+	GList *items = NULL;
+	gboolean empty_attrs = (attrs == NULL);
+	HTMLTextPangoInfo *pi = NULL;
+
+	/* printf ("itemize + glyphs\n"); */
+
+	if (empty_attrs) {
+		attrs = pango_attr_list_new ();
+		attr = pango_attr_font_desc_new (desc);
+		attr->start_index = 0;
+		attr->end_index = bytes;
+		pango_attr_list_insert (attrs, attr);
+	}
+	items = pango_itemize (gtk_widget_get_pango_context (painter->widget), text, 0, bytes, attrs, NULL);
+	if (empty_attrs)
+		pango_attr_list_unref (attrs);
+
+	if (items && items->data) {
+		PangoItem *item;
+		GList *il;
+		const gchar *end;
+		gint i = 0;
+
+		pi = html_text_pango_info_new (g_list_length (items));
+
+		*glyphs = NULL;
+		for (il = items; il; il = il->next) {
+			item = (PangoItem *) il->data;
+			pi->entries [i].item = item;
+			end = g_utf8_offset_to_pointer (text, item->num_chars);
+			*glyphs = html_get_glyphs_non_tab (*glyphs, item, i, text, end - text, item->num_chars);
+			text = end;
+			i ++;
+		}
+		*glyphs = g_list_reverse (*glyphs);
+		g_list_free (items);
+	} else
+		*glyphs = NULL;
+
+	return pi;
 }
 
 void
-html_painter_set_magnification (HTMLPainter *painter, GtkHTML *html, gdouble magnification)
+html_painter_glyphs_destroy (GList *glyphs)
 {
-	if (painter->magnification != magnification) {
-		if (magnification != 1.0) {
-			if (!painter->mag_fm) {
-				GtkHTMLClassProperties *prop;
-				painter->mag_fm = html_font_manager_new (gtk_type_class (GTK_OBJECT_TYPE (painter)));
-				prop = GTK_HTML_CLASS (GTK_OBJECT (html)->klass)->properties;
+	GList *l;
 
-				html_font_manager_set_default (painter->mag_fm,
-							       prop->font_var,      prop->font_fix,
-							       prop->font_var_size, prop->font_var_points,
-							       prop->font_fix_size, prop->font_fix_points);
-			}
-			html_font_manager_set_magnification (painter->mag_fm, magnification);
-		} else {
-			if (painter->mag_fm) {
-				html_font_manager_destroy (painter->mag_fm);
-				painter->mag_fm = NULL;
-			}
-		}
-		painter->magnification = magnification;
-	}
+	for (l = glyphs; l; l = l->next->next)
+		pango_glyph_string_free ((PangoGlyphString *) l->data);
+	g_list_free (glyphs);
 }
